from flexmock import flexmock
from unittest import TestCase
from redis.client import StrictRedis
from redis.exceptions import ResponseError
from django_cluster_redis.cache import ClusterRedis


class FakeConnection:
    host = None

    def __init__(self, host):
        self.host = host


class CacheTestCase(TestCase):
    def setUp(self):
        super(CacheTestCase, self).setUp()
        self.base_redis_mock = flexmock(StrictRedis)
        self.redis = ClusterRedis()

    def test_intercept_action(self):
        self.base_redis_mock.should_receive('get').with_args('test').and_return('asdf')
        value = self.redis.get('test')
        self.assertEqual(value, 'asdf')

    def test_follows_move(self):
        self.base_redis_mock.should_receive('set').with_args('test', 'asdf')\
            .and_raise(ResponseError, 'MOVED 9134 123.test.ip.321:6379').ordered().once()
        self.base_redis_mock.should_receive('set').with_args('test', 'asdf')\
            .and_return('foo').ordered().once()

        mock_pool = flexmock(connection_kwargs={}, _available_connections=[])
        mock_pool.should_receive('make_connection').with_args().and_return('fake connection')
        self.redis.connection_pool = mock_pool
        value = self.redis.set('test', 'asdf')
        self.assertEqual(value, 'foo')
        self.assertEqual(mock_pool._available_connections, ['fake connection'])

    def test_reuses_existing_connection(self):
        self.base_redis_mock.should_receive('set').with_args('test', 'asdf')\
            .and_raise(ResponseError, 'MOVED 9134 123.test.ip.321:6379').ordered().once()
        self.base_redis_mock.should_receive('set').with_args('test', 'asdf')\
            .and_return('foo').ordered().once()

        conn_1 = FakeConnection('asdf')
        conn_2 = FakeConnection('123.test.ip.321')
        conn_3 = FakeConnection('fdsa')
        fake_connections = [conn_1, conn_2, conn_3]
        mock_pool = flexmock(connection_kwargs={}, _available_connections=fake_connections)
        self.redis.connection_pool = mock_pool
        value = self.redis.set('test', 'asdf')
        self.assertEqual(value, 'foo')
        self.assertEqual(mock_pool._available_connections, [conn_1, conn_3, conn_2])



from setuptools import find_packages, setup

setup(
    name='django-cluster-redis',
    version='1.0.1',
    description='django cluster redis',
    long_description='This is an AWS ElasticCache (or similar cluster) adapter for django redis',
    keywords='django cluster redis aws elasticcache',
    url='https://github.com/deforestg/django-cluster-redis',
    author='Gabriel de Forest',
    packages=find_packages(exclude=['test']),
    install_requires=[
        'Django>=1.8.1',
        'django-redis>=4.7.0',
    ],
    include_package_data=True,
)



from redis.client import StrictRedis
from redis.exceptions import ResponseError


class ClusterRedis(StrictRedis):
    def _action(self, action_type, *args, **kwargs):
        method = getattr(super(ClusterRedis, self), action_type)
        try:
            return method(*args, **kwargs)
        except ResponseError as e:
            msg = str(e)
            if 'MOVED' not in msg:
                raise e

            # parse out the address
            host = msg.rsplit(' ')[2].rsplit(':')[0]

            self._follow_redirect(host)

            return method(*args, **kwargs)

    def _follow_redirect(self, host):
        pool = self.connection_pool

        pool.connection_kwargs['host'] = host
        available_connections = pool._available_connections
        # find existing connection to reuse - redis always pops off the last connection
        connection = [c for c in available_connections if c.host == host]

        if connection:
            connection = connection[0]
            available_connections.remove(connection)
        else:
            connection = pool.make_connection()

        available_connections.append(connection)

    def get(self, *args, **kwargs):
        return self._action('get', *args, **kwargs)

    def set(self, *args, **kwargs):
        return self._action('set', *args, **kwargs)

    def delete(self, *args, **kwargs):
        return self._action('delete', *args, **kwargs)

    def exists(self, *args, **kwargs):
        return self._action('exists', *args, **kwargs)

    def dump(self, *args, **kwargs):
        return self._action('dump', *args, **kwargs)

    def expire(self, *args, **kwargs):
        return self._action('expire', *args, **kwargs)

    def move(self, *args, **kwargs):
        return self._action('move', *args, **kwargs)

    def decr(self, *args, **kwargs):
        return self._action('decr', *args, **kwargs)

    def flushall(self, *args, **kwargs):
        return self._action('flushall', *args, **kwargs)

    def flushdb(self, *args, **kwargs):
        return self._action('flushdb', *args, **kwargs)

    def ttl(self, *args, **kwargs):
        return self._action('ttl', *args, **kwargs)

    def type(self, *args, **kwargs):
        return self._action('type', *args, **kwargs)

    def rename(self, *args, **kwargs):
        return self._action('rename', *args, **kwargs)
